/***
   NAME
     coho.c
***/

/*==========================================================================
 *			INCLUDING THE HEADER FILE
 *==========================================================================
 */
#include  "escbox.h"

/*
 *==========================================================================
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *==========================================================================
 */

#define time 		env[0]
#define	age		i_state(0)
#define	deathrate	i_const(0)


/*
 *==========================================================================
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *==========================================================================
 */

#define D0		parameter[0]
#define D1		parameter[1]
#define D2		parameter[2]
#define E               parameter[3]

#define YEAR            100.0
#define SMALL           0.01

/*
 *==========================================================================
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *==========================================================================
 */

void	UserInit( int argc, char **argv, double *env,  population *pop)
{

  return;
}

/*
 *==========================================================================
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *==========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)
{
  bpoint_no[0]=1;
  
  return;
}

/*==========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)
{
  bpoints[0][0][age]=0.0; 
  
  return;
}


/*
 *==========================================================================
 *			SPECIFICATION OF DERIVATIVES
 *==========================================================================
 */

void	Gradient(double *env,     population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)
{
  register int		i;
 
  for(i=0; i<cohort_no[0]; i++)			/* Determine death rates  */
    {
      if(pop[0][i][age]<1.0*YEAR)
	popIDcard[0][i][deathrate] = D0;
      else if(pop[0][i][age]<2.0*YEAR)
	popIDcard[0][i][deathrate] = D1;
      else if(pop[0][i][age]<3.0*YEAR)
	popIDcard[0][i][deathrate] = D2;
    }
  
  for(i=0; i<cohort_no[0]; i++)			/* The derivatives for all  */
    {						/* internal cohorts	    */
      popgrad[0][i][number] = -popIDcard[0][i][deathrate]*pop[0][i][number];
      popgrad[0][i][age]    = 1.0;
    }
  envgrad[0] = 1.0;

  return;
}



/*
 *==========================================================================
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *==========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)
{
  double                date, juveniles;
  register int		i;

  /* What is the date? */
  date = fmod(time, YEAR);
  if (date != 0.0)
    return;                
  /* Return if within season, otherwise opportunity to reproduce */
 
  /* semelparity: 3 year old individuals reproduce and die */
  for (i=0, juveniles=0.0; i<cohort_no[0]; i++)
    if (pop[0][i][age]>(3.0*YEAR-SMALL))
      {
	fprintf(stderr, "reproduction at time %f\n", time);
	juveniles += E*pop[0][i][number];
	pop[0][i][number] = 0.0;
      }  

  ofs[0][0][number]             = juveniles;
  ofs[0][0][age]                = 0.0;
  ofsIDcard[0][0][deathrate]    = D0;
    
  return;
}



/*
 *==========================================================================
 *			SPECIFICATION OF OUTPUT VARIABLES
 *==========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)
{

  double                totpop, age0, age1, age2;
  register int		i;	
  
  for(i=0, totpop=0.0; i<cohort_no[0]; i++)
    {
      totpop += pop[0][i][number];
    }	

  /* count individuals in three age classes */
  age0=0.0;
  age1=0.0;
  age2=0.0;
  for(i=0; i<cohort_no[0]; i++)		      
    {
      if(pop[0][i][age] < 1.0*YEAR-SMALL)
	age0=pop[0][i][number];
      else if(pop[0][i][age] < 2.0*YEAR-SMALL)
	age1=pop[0][i][number];
      else if(pop[0][i][age] < 3.0*YEAR-SMALL)
	age2=pop[0][i][number];
    }
  
  output[0] = totpop;
  output[1] = age0;
  output[2] = age1;
  output[3] = age2;
  output[4] = cohort_no[0];
  
  return;
}
/*==========================================================================*/
